#!/bin/bash
#

# buildRoot.sh <e2|mu2e:e2|nu:e2|e3|mu2e:e3|nu:e3> <debug|opt|prof> [tar] [source]

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <e2|mu2e:e2|nu:e2|e3|mu2e:e3|nu:e3> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

make_tarball()
{
    echo ""
    set -x
    cd ${PRODUCTS}
    tarname=${package}-${pkgdotver}-${OS}-${plat}-${qualdir}.tar.bz2
    tar cjf ${tarname} \
       ${package}/${pkgver}.version/${flvr}_${qualver} \
       ${package}/${pkgver}/ups \
       ${extras} \
       ${package}/${pkgver}/${flvr}-${qualdir}
    cd ${thisdir}
    set +x
    exit 0
}

make_source_tarball()
{
    set -x
    cd ${PRODUCTS}
    tar cjf ${package}-${pkgdotver}-source.tar.bz2 \
       ${package}/${pkgver}/*.sh \
       ${package}/${pkgver}/ups \
       ${source_extras} \
       ${package}/${pkgver}/source
    cd ${thisdir}
    set +x
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`

    if [ "${basequal}" = "e2" ] 
    then
	shortqual=e
	cqual=e2
    elif [ "${basequal}" = "nu:e2" ] 
    then
	shortqual=nu
	cqual=e2
    elif [ "${basequal}" = "mu2e:e2" ] 
    then
	shortqual=mu2e
	cqual=e2
    elif [ "${basequal}" = "e3" ] 
    then
	shortqual=e
	cqual=e3
    elif [ "${basequal}" = "nu:e3" ] 
    then
	shortqual=nu
	cqual=e3
    elif [ "${basequal}" = "mu2e:e3" ] 
    then
	shortqual=mu2e
	cqual=e3
    else
       echo "ERROR: unrecognized qualifier"
       usage
       exit 1
    fi
}

check_with_lsb_release ()
{
   plat=`uname -p`
   if [ `lsb_release -d | cut -f2 | cut  -f1 -d" "` = "Ubuntu" ]
   then
      OS1=ub
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      plat=`uname -m`
   elif [ ${OS1} = "Linux" ]
   then
      OSnum=`lsb_release -r | cut -f2 | cut -f1 -d"."`
      OS=${OS1}${OSnum}
      # Scientific Linux - slf should work
      if [ `lsb_release -d | cut  -f3 -d" "` = "SL" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux Fermi
      elif [ `lsb_release -d | cut  -f3 -d" "` = "SLF" ]
      then
	 OS=slf${OSnum}
      #
      elif [ `lsb_release -i | cut -f2` = "ScientificFermi" ]
      then
	 OS=slf${OSnum}
      # pretend that SL6 is the same as SLF6
      elif [ `lsb_release -i | cut -f2` = "Scientific" ]
      then
	 OS=slf${OSnum}
      # pretend that CentOS is SLF
      elif [ `lsb_release -i | cut -f2` = "CentOS" ]
      then
	 OS=slf${OSnum}
      # pretend that RedHatEnterpriseServer is SLF
      elif [ `lsb_release -i | cut -f2` = "RedHatEnterpriseServer" ]
      then
	 OS=slf${OSnum}
      # Scientific Linux CERN
      elif [ `lsb_release -d | cut  -f4 -d" "` = "SLC" ]
      then
	 OS=slc${OSnum}
      elif [ `lsb_release -d | cut  -f4 -d" "` = "LTS" ]
      then
	 OS=slf${OSnum}
      # unrecognized - pretend that this is SLF
      else
	 OS=slf${OSnum}
      fi
   fi
}

default_names ()
{
      OSnum1=`uname -r | cut -f1 -d"."`
      OSnum2=`uname -r | cut -f2 -d"."`
      OS=${OS1}${OSnum1}${OSnum2}
      plat=`uname -p`
}

check_linux ()
{
   plat=`uname -p`
   if [ -e /etc/system-release-cpe ]
   then
      OSnum=`cat /etc/system-release-cpe | cut -f5 -d":" |  cut -f1 -d"."`
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
      else
         OS=slf${OSnum}
      fi
   elif [ -e /etc/redhat-release ]
   then
      if [ `cat /etc/redhat-release | cut  -f4 -d" "` = "SLC" ]
      then
         OS=slc${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f6 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "Fermi" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      elif [ `cat /etc/redhat-release | cut  -f3 -d" "` = "SLF" ]
      then
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      else
         OS=slf${OSnum}
         OSnum=`cat /etc/redhat-release | cut -f5 -d" " |  cut -f1 -d"."`
      fi
   else
      default_names
   fi
}

get_platform_name()
{
    source ${PRODUCTS}/setup

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
	plat=`uname -m`
	OSnum=`uname -r | cut -f1 -d"."`
	macos=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1,2`
	macos1=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 1`
	macos2=`system_profiler -xml -detailLevel mini SPSoftwareDataType | grep Mac | cut -d ' ' -f 4 | cut -d . -f 2`
	#OS=${OS1}${OSnum}
	OS=mac${macos1}${macos2}
    elif [ "${flvr0}" = "Linuxppc" ]
    then
       OS=slf5
       plat=`uname -p`
    elif [ -e /usr/bin/lsb_release ]
    then
       check_with_lsb_release
    elif [ ${OS1} = "Linux" ]
    then
       check_linux
    else
       default_names
    fi

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${PRODUCTS}/${package}/${pkgver}/${flvr}-${qualdir}
    sourcedir=${PRODUCTS}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${cqual}" = "e2" ] ||  [ "${cqual}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${cqual}" = "e3" ] ||  [ "${cqual}" = "gcc473" ]
    then
       gccdotver=4.7
       gccver=v4_7_3
    elif [ "${cqual}" = "gcc46" ]
    then
       gccdotver=4.6
       gccver=v4_6_1
    fi
}

check_gcc()
{
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ "${gccshortver}" = "${gccdotver}" ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    check_gcc
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

basequal=${1}
extraqual=${2}
maketar=${3}
sourcetar=${4}

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=root
pkgver=v5_34_07
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
source_extras=${package}/${pkgver}/MyRules* 
pkgtarfile=root_v${pkgdotver}.source.tar.gz
patchfile=fermi_root_5_34.patch

get_this_dir

if [ "${sourcetar}" = "source" ]
then
  make_source_tarball
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# use ROOTSYS/MyRules.mk
if [ "${extraqual}" = "opt" ]
then
    rules=MyRules_opt.mk
elif [ "${extraqual}" = "debug" ]
then
    extracommand="--build=debug"
    rules=MyRules_debug.mk
elif [ "${extraqual}" = "prof" ]
then
    rules=MyRules_prof.mk
fi

# declare now so we can setup
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

declare_product

# setup root
setup ${package} ${pkgver} -q ${fullqual}

check_gcc

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ "${shortqual}" = "nu" ] 
then
    if [ -z ${POSTGRESQL_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${POSTGRESQL_VERSION}"
    fi

    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${pkgdir}
tar xzf ${sourcedir}/${pkgtarfile}

# get the extra rules 
if [ -n ${rules} ]
then
   cp ${ROOT_DIR}/${rules} ${pkgdir}/${package}/MyRules.mk
fi
if [ "${cqual}" = "e2" ] || [ "${cqual}" = "e3" ]
then
  c11="--enable-c++11"
  echo "CXXFLAGS += -std=c++11 -Wno-deprecated-declarations" >> ${pkgdir}/root/MyRules.mk
  echo "NOOPT += -std=c++11 -Wno-deprecated-declarations" >> ${pkgdir}/root/MyRules.mk
fi

set -x

cd ${pkgdir}/root

# apply Fermi patches
patch -b -p0 < ${sourcedir}/${patchfile}

if [ "${shortqual}" = "nu" ]
then
./configure ${extracommand} \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
--with-pythia6-libdir=${PYLIB} \
--with-pgsql-incdir=${POSTGRESQL_FQ_DIR}/include \
--with-pgsql-libdir=${POSTGRESQL_FQ_DIR}/lib \
--with-mysql-incdir=${MYSQL_CLIENT_FQ_DIR}/include \
--with-mysql-libdir=${MYSQL_CLIENT_FQ_DIR}/lib
elif [ "${shortqual}" = "mu2e" ]
then
./configure ${extracommand} \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
else
./configure ${extracommand} \
--enable-gdml ${c11} \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--disable-xrootd \
--enable-minuit2 \
--disable-xft \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib
fi

make -j3
# now install in ROOTSYS
make install 

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual}
echo "${package} is installed at ${ROOTSYS}"

# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   make_tarball
fi

exit 0
